@@ -14,6 +14,7 @@ import mmpm.utils
 import mmpm.consts
 import mmpm.core
 import mmpm.models
+import mmpm.mmpm
 
 MagicMirrorPackage = mmpm.models.MagicMirrorPackage
 get_env: Callable = mmpm.utils.get_env
@@ -35,8 +36,8 @@ resources: dict = {
 
 CORS(app)
 
+# a quick wrapper lambda for adding a prefix to the urls, there's more "official" ways of doing this, but I don't care to
 api = lambda path: f'/api/{path}'
-
 _packages_ = mmpm.core.load_packages()
 
 
@@ -68,6 +69,15 @@ def __deserialize_selected_packages__(rqst, key: str = 'selected-packages') -> L
 
 @app.after_request
 def after_request(response: Response) -> Response:
+    '''
+    Appends extra headers after each api request is sent to the server
+
+    Parameters:
+        response (flask.Response): the response object being returned to the frontend
+
+    Returns
+        response (flask.Response): the modified response object with new headers attached
+    '''
     response.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
     response.headers["Pragma"] = "no-cache"
     response.headers["Expires"] = "0"
@@ -83,35 +93,90 @@ def static_proxy(path):
 
 @app.route('/', methods=[mmpm.consts.GET, mmpm.consts.POST, mmpm.consts.DELETE])
 def root() -> str:
+    '''
+    Returns the index.html page from the template folder as requested by the
+    frontend back to the root url
+
+    Parameters:
+        None
+
+    Returns:
+        index (str): the index.html content as a string
+    '''
     return render_template('index.html')
 
 
 @app.errorhandler(500)
 def server_error(error) -> Response:
+    '''
+    Sends back a verbose error message to the frontend alerting an error occurred
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the response object containing the error message
+    '''
     return Response(f'An internal error occurred [{__name__}.py]: {error}', status=500)
 
 
 #  -- START: PACKAGES --
 @app.route(api('packages/marketplace'), methods=[mmpm.consts.GET])
 def packages_marketplace() -> Response:
+    '''
+    Sends back all of the available packages in the MMPM/MagicMirror database
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the response object containing all the available packages
+    '''
     mmpm.utils.log.info('Sending all marketplace packages')
     return Response(json.dumps(_packages_, default=lambda pkg: pkg.serialize_full()))
 
 
 @app.route(api('packages/installed'), methods=[mmpm.consts.GET])
 def packages_installed() -> Response:
+    '''
+    Sends back all of the installed packages
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the response object containing all the installed packages
+    '''
     mmpm.utils.log.info('Sending all installed packages')
     return Response(json.dumps(mmpm.core.get_installed_packages(_packages_), default=lambda pkg: pkg.serialize_full()))
 
 
 @app.route(api('packages/external'), methods=[mmpm.consts.GET])
 def packages_external() -> Response:
+    '''
+    Sends back all the external packages created by the user to the frontend
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the response object containing all the external packages
+    '''
     mmpm.utils.log.info('Sending all external packages')
     return Response(json.dumps(mmpm.core.load_external_packages(), default=lambda pkg: pkg.serialize_full()))
 
 
 @app.route(api('packages/install'), methods=[mmpm.consts.POST])
 def packages_install() -> Response:
+    '''
+    Installs the selected packages on the user's device
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the result following the installation, success or failure
+    '''
     selected_packages: List[MagicMirrorPackage] = __deserialize_selected_packages__(request)
     failures: List[dict] = []
 
@@ -129,6 +194,15 @@ def packages_install() -> Response:
 
 @app.route(api('packages/remove'), methods=[mmpm.consts.POST])
 def packages_remove() -> Response:
+    '''
+    Removes the selected packages from the user's device
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the result following the removal, success or failure
+    '''
     # not bothering with serialization since the directory is already included in the request
     selected_packages: List[dict] = request.get_json(force=True)['selected-packages']
     failures: List[dict] = []
@@ -151,6 +225,15 @@ def packages_remove() -> Response:
 
 @app.route(api('packages/upgrade'), methods=[mmpm.consts.POST])
 def packages_upgrade() -> Response:
+    '''
+    Upgrades the selected packages on the user's device
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the result following the package upgrade(s), success or failure
+    '''
     selected_packages: List[MagicMirrorPackage] = __deserialize_selected_packages__(request)
     mmpm.utils.log.info(f'Request to upgrade {selected_packages}')
 
@@ -168,6 +251,15 @@ def packages_upgrade() -> Response:
 
 @app.route(api('packages/update'), methods=[mmpm.consts.GET])
 def packages_update() -> Response:
+    '''
+    Checks for package/application updates within the users MMPM/MagicMirror installation
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the result of the update check, success or failure as a boolean
+    '''
     try:
         mmpm.core.check_for_package_updates(_packages_)
         mmpm.core.check_for_mmpm_updates()
@@ -180,6 +272,15 @@ def packages_update() -> Response:
 
 @app.route(api('packages/upgradeable'), methods=[mmpm.consts.GET])
 def packages_upgradeable() -> Response:
+    '''
+    Sends back a JSON object detailing which packages/applications have available upgrades
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the object containing which packages/applications have available upgrades
+    '''
     mmpm.utils.log.info('Request to get upgradeable packages')
     available_upgrades: dict = mmpm.core.get_available_upgrades()
 
@@ -197,6 +298,15 @@ def packages_upgradeable() -> Response:
 #  -- START: EXTERNAL PACKAGES --
 @app.route(api('external-packages/add'), methods=[mmpm.consts.POST])
 def external_packages_add() -> Response:
+    '''
+    Adds an External Package to the user's database
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the success or failure of adding the external package to the database
+    '''
     package: dict = request.get_json(force=True)['external-package']
 
     failures: List[dict] = []
@@ -214,6 +324,15 @@ def external_packages_add() -> Response:
 
 @app.route(api('external-packages/remove'), methods=[mmpm.consts.DELETE])
 def external_packages_remove() -> Response:
+    '''
+    Removes an External Package from the user's database
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the success or failure of removing the external package from the database
+    '''
     selected_packages: List[MagicMirrorPackage] = __deserialize_selected_packages__(request, 'external-packages')
     mmpm.utils.log.info('Request to remove external sources')
 
@@ -249,6 +368,15 @@ def external_packages_remove() -> Response:
 #  -- START: MAGICMIRROR --
 @app.route(api('magicmirror/config'), methods=[mmpm.consts.GET, mmpm.consts.POST])
 def magicmirror_config() -> Response:
+    '''
+    Retrieves the user's MagicMirror config.js, and sends file contents to frontend
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the file contents
+    '''
     MAGICMIRROR_CONFIG_DIR: str = os.path.join(get_env(mmpm.consts.MMPM_MAGICMIRROR_ROOT_ENV), 'config')
     MAGICMIRROR_CONFIG_FILE: str = os.path.join(MAGICMIRROR_CONFIG_DIR, 'config.js')
 
@@ -279,6 +407,15 @@ def magicmirror_config() -> Response:
 
 @app.route(api('magicmirror/custom-css'), methods=[mmpm.consts.GET, mmpm.consts.POST])
 def magicmirror_custom_css() -> Response:
+    '''
+    Retrieves the user's MagicMirror custom.css, and sends file contents to frontend
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the file contents
+    '''
     MAGICMIRROR_CUSTOM_DIR: str = os.path.join(get_env(mmpm.consts.MMPM_MAGICMIRROR_ROOT_ENV), 'custom')
     MAGICMIRROR_CUSTOM_CSS_FILE: str = os.path.join(MAGICMIRROR_CUSTOM_DIR, 'custom.css')
 
@@ -318,7 +455,9 @@ def magicmirror_start() -> Response:
         None
 
     Returns:
-        bool: True if the command was called, False it appears that MagicMirror is currently running
+        response (flask.Response): response containing boolean of True if the
+        command was called, False it appears that MagicMirror is currently
+        running
     '''
     # there really isn't an easy way to capture return codes for the background
     # process
@@ -343,7 +482,8 @@ def magicmirror_restart() -> Response:
         None
 
     Returns:
-        bool: Always True only as a signal the process was called
+        response(flask.Response): response object containing boolean, which is
+        always True only as a signal the process was called
     '''
     # same issue as the start-magicmirror api call
     mmpm.core.restart_magicmirror()
@@ -359,7 +499,8 @@ def magicmirror_stop() -> Response:
         None
 
     Returns:
-        bool: Always True only as a signal the process was called
+        response (flask.Response): response containing boolean that is always
+        True only as a signal the process was called
     '''
     # same sort of issue as the start-magicmirror call
     mmpm.core.stop_magicmirror()
@@ -368,6 +509,17 @@ def magicmirror_stop() -> Response:
 
 @app.route(api('magicmirror/upgrade'), methods=[mmpm.consts.GET])
 def magicmirror_upgrade() -> Response:
+    '''
+    Upgrades the MagicMirror application from the Control Center of the GUI
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): response containing an error message, if any
+        error was encountered. If no error was generated, an empty string is
+        returned
+    '''
     mmpm.utils.log.info('Request to upgrade MagicMirror')
     mmpm.utils.log.info('Finished installing')
 
@@ -378,8 +530,20 @@ def magicmirror_upgrade() -> Response:
 
     return Response(json.dumps({'error': error}))
 
+
 @app.route(api('magicmirror/install-mmpm-module'))
 def magicmirror_install_mmpm_module() -> Response:
+    '''
+    Install the MMPM MagicMirror module in the user's module path
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): response containing an error message, if any
+        error was encountered. If no error was generated, an empty string is
+        returned
+    '''
     return Response(json.dumps({'error': mmpm.core.install_mmpm_as_magicmirror_module(assume_yes=True)}))
 #  -- END: MAGICMIRROR --
 
@@ -394,9 +558,9 @@ def raspberrypi_restart() -> Response:
         None
 
     Returns:
-        success (bool): If the command fails, False is returned. If success, the return will never reach the interface
+        response (flask.Response): If the command fails, False is returned. If
+        success, the return will never reach the interface
     '''
-
     mmpm.utils.log.info('Restarting RaspberryPi')
     mmpm.core.stop_magicmirror()
     error_code, _, _ = mmpm.utils.run_cmd(['sudo', 'reboot'])
@@ -413,9 +577,9 @@ def raspberrypi_stop() -> Response:
         None
 
     Returns:
-        success (bool): If the command fails, False is returned. If success, the return will never reach the interface
+        response (flask.Response): If the command fails, False is returned. If success,
+        the return will never reach the interface
     '''
-
     mmpm.utils.log.info('Shutting down RaspberryPi')
     # if success, we'll never get the response, but we'll know if it fails
     mmpm.core.stop_magicmirror()
@@ -425,6 +589,17 @@ def raspberrypi_stop() -> Response:
 
 @app.route(api('raspberrypi/rotate-screen'), methods=[mmpm.consts.POST])
 def raspberrypi_rotate_screent() -> Response:
+    '''
+    Rotate the RaspberryPi screen by the number of degrees provided.
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): response containing an error message, if any
+        error was encountered. If no error was generated, an empty string is
+        returned
+    '''
     degrees: int = request.get_json(force=True)['degrees']
     return Response(json.dumps({'error': mmpm.core.rotate_raspberrypi_screen(degrees, assume_yes=True)}))
 #  -- END: RASPBERRYPI --
@@ -433,6 +608,15 @@ def raspberrypi_rotate_screent() -> Response:
 #  -- START: MMPM --
 @app.route(api('mmpm/download-logs'), methods=[mmpm.consts.GET])
 def download_log_files() -> Response:
+    '''
+    Compresses the log files generated by MMPM, and sends them back to the frontend for the user to download.
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the log files as a single zip file
+    '''
     os.chdir('/tmp')
     today = datetime.datetime.now()
     zip_file_name = f'mmpm-logs-{today.year}-{today.month}-{today.day}'
@@ -440,8 +624,18 @@ def download_log_files() -> Response:
     return send_file(f'/tmp/{zip_file_name}.zip', attachment_filename='{}.zip'.format(zip_file_name), as_attachment=True)
 
 
+# this is stupid and should be condensed
 @app.route(api('mmpm/environment-vars'), methods=[mmpm.consts.GET])
 def mmpm_environment_vars() -> Response:
+    '''
+    Sends the MMPM environment variables as json object to the frontend
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): a JSON object containing the MMPM environment variables
+    '''
     env_vars: dict = {}
 
     with open(mmpm.consts.MMPM_ENV_FILE, 'r') as env:
@@ -455,6 +649,19 @@ def mmpm_environment_vars() -> Response:
 
 @app.route(api('mmpm/environment-vars-file'), methods=[mmpm.consts.GET, mmpm.consts.POST])
 def mmpm_environment_vars_file() -> Response:
+    '''
+    If the frontend sends a GET request, the MMPM environment variables
+    contents is sent back. If the request method is a POST, then the updated
+    file contents are written to the mmpm-env.json file.
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the MMPM environment variables file contents
+        if it is a GET, otherwise the file contents are written to the user's
+        mmpm-env.json file
+    '''
     if request.method == mmpm.consts.GET:
         return send_file(mmpm.consts.MMPM_ENV_FILE, attachment_filename='mmpm-env.json')
 
@@ -468,3 +675,17 @@ def mmpm_environment_vars_file() -> Response:
         return Response(json.dumps(False))
 
     return Response(json.dumps(True))
+
+
+@app.route(api('mmpm/version'), methods=[mmpm.consts.GET])
+def mmpm_version() -> Response:
+    '''
+    Sends the MMPM version number
+
+    Parameters:
+        None
+
+    Returns:
+        response (flask.Response): the MMPM version number
+    '''
+    return Response(json.dumps({'version': mmpm.mmpm.__version__}))
